// UFXLibDnTest.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "UFXUtilFunctions.h"

// 2 bits for I2S audio sampling frequency
#define I2S_FS_AUTO      0
#define I2S_FS_FORCE_48  1
#define I2S_FS_FORCE_44  2

typedef enum {
    AUDIO_SPDIF = 0,
    AUDIO_I2S0 = 1,
    AUDIO_I2S1 = 2,
    AUDIO_AUTO = 3
} REGISTRY_AUDIO;

#define DC_PORT_HDMI_DVI	3UL
#define DC_PORT_HDMI_V		4UL	// video-only HDMI
#define DC_PORT_HDMI_AV		5UL	// audio & video HDMI
#define DC_PORT_HDMI_AV_I2S	6UL	// audio I2S    & video HDMI

typedef UFXLIB_RESULT (__cdecl *LIBINIT)();
typedef UFXLIB_RESULT (__cdecl *LIBDEINIT)();

typedef UFXLIB_RESULT (__cdecl *ERASEEPROM)();
typedef UFXLIB_RESULT (__cdecl *CALCEEPROMSIZE)(DWORD *pdwSize );
typedef UFXLIB_RESULT (__cdecl *WRITEEEPROM)(UCHAR *pucBuf, DWORD dStartAdd, DWORD dwLen );
typedef UFXLIB_RESULT (__cdecl *READEEPROM)(UCHAR *pucBuf, DWORD dStartAdd, DWORD dwLen );

void EepromTesting()
{	
	int iResult ;
	FARPROC lpfnGetProcessID;
	/* get handle to dll */ 
	HINSTANCE hGetProcIDDLL = NULL;
	hGetProcIDDLL = LoadLibrary(L"ufxutil.dll"); 
	if(NULL == hGetProcIDDLL)
	{
		printf("Failed to load ufxutil.dll library \n");
		return;
	}

	HMODULE hndl = NULL; 
	hndl = GetModuleHandle(L"ufxutil.dll");

	LIBINIT pUFXLibInit;
	LIBDEINIT pUFXLibDeInit;

	/* get pointer to the function in the dll*/ 
	lpfnGetProcessID = GetProcAddress(hndl,"UFXLibInit"); 
	pUFXLibInit = LIBINIT(lpfnGetProcessID);
	
	lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXLibDeInit"); 						
	pUFXLibDeInit = LIBDEINIT(lpfnGetProcessID);

	/* Init Environment */
	iResult = pUFXLibInit();
	if(UFX_LIB_MORE_THAN_ONE_DEV == iResult)
	{
		printf("UFX_LIB_MORE_THAN_ONE_DEV \n");
		return;
	}
	else if(UFX_FRM_WRK_INIT_FAIL == iResult)
		printf("UFX_FRM_WRK_INIT_FAIL\n");
	else if(UFX_HDWRE_INIT_FAIL == iResult)
		printf("UFX_HDWRE_INIT_FAIL \n");
	
	printf("********* EEPROM Function testing ******** \n\n");

	/**************************** Reading/Writing to/From EEPROM: Ends **************/ 
	do{
		int iOper = 0;
		printf("-------- EEPROM functions Test Menu -----------\n");
		printf("\t1. Calculate EEPROM Size \n");
		printf("\t2. Write in EEPROM \n");
		printf("\t3. Read from EEPROM \n");
		printf("\t4. Erase EEPROM \n");
		//printf("\t5. Read EEPROM content\n");
		printf("\t5. Exit \n");
		scanf("%d",&iOper);

		switch(iOper)
		{
			case 1:
				{
					DWORD dErpromSize = 0;
					CALCEEPROMSIZE pUFXCalcEepromSize;
					pUFXCalcEepromSize = (CALCEEPROMSIZE)GetProcAddress(HMODULE (hGetProcIDDLL),"UFXCalcEepromSize"); 						
					iResult = pUFXCalcEepromSize(&dErpromSize);		
					if(UFX_LIB_SUCCESS == iResult)	
						printf("EEPROM Size = %d \n", dErpromSize);
					else if(UFX_LIB_MORE_THAN_ONE_DEV == iResult)
						printf("UFX_LIB_MORE_THAN_ONE_DEV \n");
					else if(UFX_FRM_WRK_INIT_FAIL == iResult)
						printf("UFX_FRM_WRK_INIT_FAIL\n");
					else if(UFX_HDWRE_INIT_FAIL == iResult)
						printf("UFX_HDWRE_INIT_FAIL \n");				
				}
				break;

			case 2:
				{
					int iStrlen = 0; 
					char *puBuf = "RAWDATA:12345678900123456";
					iStrlen = strlen(puBuf); 

					WRITEEEPROM pUFXLibWriteEEprom;
					lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXLibWriteEEprom"); 						
					pUFXLibWriteEEprom = WRITEEEPROM(lpfnGetProcessID);

					iResult = pUFXLibWriteEEprom((UCHAR*)puBuf, 0, iStrlen);
					if(UFX_LIB_SUCCESS == iResult)	
						printf("String written to EEPROM = '%s' \n", puBuf);
					else if(UFX_LIB_MORE_THAN_ONE_DEV == iResult)
						printf("UFX_LIB_MORE_THAN_ONE_DEV \n");
					else if(UFX_FRM_WRK_INIT_FAIL == iResult)
						printf("UFX_FRM_WRK_INIT_FAIL\n");
					else if(UFX_HDWRE_INIT_FAIL == iResult)
						printf("UFX_HDWRE_INIT_FAIL \n");
					else if(UFX_DEV_COM_FAIL == iResult)
						printf("UFX_DEV_COM_FAIL \n");	
					else if(UFX_LIB_FAILURE == iResult)
						printf("UFX_LIB_FAILURE \n");
					else if(UFX_INVALID_ARG == iResult)
						printf("UFX_INVALID_ARG \n");
					else if(UFX_EEPROM_OUT_OF_BOUND == iResult)
						printf("UFX_EEPROM_OUT_OF_BOUND \n");
					else if(UFX_INVALID_EEPROM_ADD == iResult)
						printf("UFX_INVALID_EEPROM_ADD \n");
				}
				break;

			case 3:
				{
					READEEPROM pUFXLibReadEeprom;
					lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXLibReadEeprom"); 						
					pUFXLibReadEeprom = READEEPROM(lpfnGetProcessID);

					char *puEEBuf = (char*)malloc(100 *sizeof(char));
					if(puEEBuf == NULL)
					{
						pUFXLibDeInit();
						return;
					}
					char *puBuf = "RAWDATA:12345678900123456";
					int iStrlen = strlen(puBuf); 
					memset((void*)puEEBuf, 0x00, 90);

					iResult = pUFXLibReadEeprom((UCHAR*)puEEBuf, 0, iStrlen);
					if(UFX_LIB_SUCCESS == iResult)	
						printf("String read from EEPROM = '%s' \n", puEEBuf);
					else if(UFX_LIB_MORE_THAN_ONE_DEV == iResult)
						printf("UFX_LIB_MORE_THAN_ONE_DEV \n");
					else if(UFX_FRM_WRK_INIT_FAIL == iResult)
						printf("UFX_FRM_WRK_INIT_FAIL\n");
					else if(UFX_HDWRE_INIT_FAIL == iResult)
						printf("UFX_HDWRE_INIT_FAIL \n");
					else if(UFX_DEV_COM_FAIL == iResult)
						printf("UFX_DEV_COM_FAIL \n");	
					else if(UFX_LIB_FAILURE == iResult)
						printf("UFX_LIB_FAILURE \n");
					else if(UFX_INVALID_ARG == iResult)
						printf("UFX_INVALID_ARG \n");
					else if(UFX_EEPROM_OUT_OF_BOUND == iResult)
						printf("UFX_EEPROM_OUT_OF_BOUND \n");
					else if(UFX_INVALID_EEPROM_ADD == iResult)
						printf("UFX_INVALID_EEPROM_ADD \n");
				
					free(puEEBuf);
				}
				break;

			case 4:
				{
					ERASEEPROM pUFXLibEraseEeprom;
					lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXLibEraseEeprom"); 						
					pUFXLibEraseEeprom = ERASEEPROM(lpfnGetProcessID);
					iResult = pUFXLibEraseEeprom();
					if(UFX_LIB_SUCCESS == iResult)	
						printf("Erase EEPROM success \n");
					else if(UFX_LIB_MORE_THAN_ONE_DEV == iResult)
						printf("UFX_LIB_MORE_THAN_ONE_DEV \n");
					else if(UFX_FRM_WRK_INIT_FAIL == iResult)
						printf("UFX_FRM_WRK_INIT_FAIL\n");
					else if(UFX_HDWRE_INIT_FAIL == iResult)
						printf("UFX_HDWRE_INIT_FAIL \n");
					else if(UFX_DEV_COM_FAIL == iResult)
						printf("UFX_DEV_COM_FAIL \n");	
					else if(UFX_LIB_FAILURE == iResult)
						printf("UFX_LIB_FAILURE \n");
					else if(UFX_INVALID_ARG == iResult)
						printf("UFX_INVALID_ARG \n");
					else if(UFX_EEPROM_OUT_OF_BOUND == iResult)
						printf("UFX_EEPROM_OUT_OF_BOUND \n");
					else if(UFX_INVALID_EEPROM_ADD == iResult)
						printf("UFX_INVALID_EEPROM_ADD \n");
				}
				break;

			case 5:
			case 6:
			default:
				goto END;	
				break;
		}
	}while(1);

END:
	iResult = pUFXLibDeInit();
	
	/* Release the DLL */
 	FreeLibrary(hGetProcIDDLL);
	
	return;
}

typedef UFXLIB_RESULT (__cdecl *GETMEMSIZE)();
typedef UFXLIB_RESULT (__cdecl *GETODTFREQ)(stMemoryParams*);
typedef UFXLIB_RESULT (__cdecl *SETODTFREQ)(stMemoryParams);
typedef stMemStatus (__cdecl *TESTDATALINES)();
typedef stMemStatus (__cdecl *TESTADDLINES)();
typedef stMemStatus (__cdecl *VERIFYFULLDRAM)();
typedef UFXLIB_RESULT (__cdecl *DRAMREAD)(UCHAR *pucBuf, DWORD dStartAdd, DWORD dwLen );
typedef UFXLIB_RESULT (__cdecl *VERIFYDRAM)(DWORD dwClkFreq, DWORD dwODTValue );
typedef UFXLIB_RESULT (__cdecl *VERIFYADDLINES)(DWORD dwClkFreq, DWORD dwODTValue );
typedef UFXLIB_RESULT (__cdecl *VERIFYDATALINES)(DWORD dwClkFreq, DWORD dwODTValue );
typedef UFXLIB_RESULT (__cdecl *WRITEDRAM)(UCHAR *pucBuf, DWORD dStartAdd, DWORD dwLen, UCHAR ucTransferType  );
typedef UFXLIB_RESULT (__cdecl *SETODTVAL)(stMemoryParams stMemObj );

void DramTesting()
{	
	stMemStatus stMemStatusobj;

	FARPROC lpfnGetProcessID;
	/* get handle to dll */ 
	HINSTANCE hGetProcIDDLL = NULL;
	hGetProcIDDLL = LoadLibrary(L"ufxutil.dll"); 
	if(NULL == hGetProcIDDLL)
	{
		printf("Failed to load ufxutil.dll library \n");
		return;
	}

	HMODULE hndl = NULL; 
	hndl = GetModuleHandle(L"ufxutil.dll");

	LIBINIT pUFXLibInit;
	LIBDEINIT pUFXLibDeInit;

	/* get pointer to the function in the dll*/ 
	lpfnGetProcessID = GetProcAddress(hndl,"UFXLibInit"); 
	pUFXLibInit = LIBINIT(lpfnGetProcessID);
	
	lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXLibDeInit"); 						
	pUFXLibDeInit = LIBDEINIT(lpfnGetProcessID);

	/* Init Environment */
	int iResult = pUFXLibInit();
	if(UFX_LIB_MORE_THAN_ONE_DEV == iResult)
	{
		printf("UFX_LIB_MORE_THAN_ONE_DEV \n");
		return;
	}
	else if(UFX_FRM_WRK_INIT_FAIL == iResult)
		printf("UFX_FRM_WRK_INIT_FAIL\n");
	else if(UFX_HDWRE_INIT_FAIL == iResult)
		printf("UFX_HDWRE_INIT_FAIL \n");
	
	printf("********* dram Function testing ******** \n\n");

	/**************************** Reading/Writing to/From EEPROM: Ends **************/ 
	do{
		int iOper = 0;
		printf("-------- EEPROM functions Test Menu -----------\n");
		printf("\t1. DRAM EEPROM Size \n");
		printf("\t2. Write in DRAM \n");
		printf("\t3. Read from DRAM \n");
		printf("\t4. Test Data Lines \n");
		printf("\t5. Test Address Lines \n");
		printf("\t6. Verify full DRAM \n");
		printf("\t7. Get ODT and Frequency Value \n");
		printf("\t8. Set ODT and Frequency Value \n");
		printf("\t9. Exit \n");
		scanf("%d",&iOper);

		switch(iOper)
		{
			case 1:	
				{
					GETMEMSIZE pUFXGetMemSize;
					pUFXGetMemSize = (GETMEMSIZE)GetProcAddress(HMODULE (hGetProcIDDLL),"UFXGetMemSize"); 						
					DWORD dwMemSize = pUFXGetMemSize();		
					printf("DRAM Size = %d \n", dwMemSize);
				}
				break;

			case 2:
				{
					SETODTVAL pUFXSetODTnFrequency;
 					lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXSetODTnFrequency"); 						
					pUFXSetODTnFrequency = SETODTVAL(lpfnGetProcessID);

			 		stMemoryParams stMemObj;
					stMemObj.dwFrequency = 1;
					stMemObj.dwODTValue = 3;
					pUFXSetODTnFrequency(stMemObj);
	 
					char *puBuf = "RAWDATA:1234567890ABCDEFGHIJKLMNOPQ";
					int iStrlen = strlen(puBuf); 

					WRITEDRAM pUFXDRAMWrite;
					lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXDRAMWrite"); 						
					pUFXDRAMWrite = WRITEDRAM(lpfnGetProcessID);

					iResult = pUFXDRAMWrite((UCHAR*)puBuf, 0, iStrlen,0);
					if(UFX_LIB_SUCCESS == iResult)	
						printf("String written to DRAM = %s \n", puBuf);
					else if(UFX_LIB_MORE_THAN_ONE_DEV == iResult)
						printf("UFX_LIB_MORE_THAN_ONE_DEV \n");
					else if(UFX_FRM_WRK_INIT_FAIL == iResult)
						printf("UFX_FRM_WRK_INIT_FAIL\n");
					else if(UFX_HDWRE_INIT_FAIL == iResult)
						printf("UFX_HDWRE_INIT_FAIL \n");
					else if(UFX_DEV_COM_FAIL == iResult)
						printf("UFX_DEV_COM_FAIL \n");	
					else if(UFX_LIB_FAILURE == iResult)
						printf("UFX_LIB_FAILURE \n");
					else if(UFX_INVALID_ARG == iResult)
						printf("UFX_INVALID_ARG \n");
					else if(UFX_DRAM_OUT_OF_BOUND == iResult)
						printf("UFX_DRAM_OUT_OF_BOUND \n");
					else if(UFX_INVALID_DRAM_ADD == iResult)
						printf("UFX_INVALID_DRAM_ADD \n");
				}
				break;

			case 3:
				{
					DRAMREAD pUFXDRAMRead;
					lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXDRAMRead"); 						
					pUFXDRAMRead = DRAMREAD(lpfnGetProcessID);
					char *puEEBuf = NULL;
					puEEBuf = (char*)malloc(100 *sizeof(char));
					if(puEEBuf == NULL)
					{
						pUFXLibDeInit();
						return;
					}

					memset((void*)puEEBuf, 0x00, 99);
					char *puBuf = "RAWDATA:1234567890ABCDEFGHIJKLMNOPQ";
					int iStrlen = strlen(puBuf); 
					iResult = pUFXDRAMRead((UCHAR*)puEEBuf, 0, iStrlen);				
					if(UFX_LIB_SUCCESS == iResult)	
						printf("String Read from DRAM = %s \n", puEEBuf);
					else if(UFX_LIB_MORE_THAN_ONE_DEV == iResult)
						printf("UFX_LIB_MORE_THAN_ONE_DEV \n");
					else if(UFX_FRM_WRK_INIT_FAIL == iResult)
						printf("UFX_FRM_WRK_INIT_FAIL\n");
					else if(UFX_HDWRE_INIT_FAIL == iResult)
						printf("UFX_HDWRE_INIT_FAIL \n");
					else if(UFX_DEV_COM_FAIL == iResult)
						printf("UFX_DEV_COM_FAIL \n");	
					else if(UFX_LIB_FAILURE == iResult)
						printf("UFX_LIB_FAILURE \n");
					else if(UFX_INVALID_ARG == iResult)
						printf("UFX_INVALID_ARG \n");
					else if(UFX_EEPROM_OUT_OF_BOUND == iResult)
						printf("UFX_EEPROM_OUT_OF_BOUND \n");
					else if(UFX_INVALID_EEPROM_ADD == iResult)
						printf("UFX_INVALID_EEPROM_ADD \n");				
					free(puEEBuf);
				}
				break;

			case 4:
				{
					TESTDATALINES pUFXMemoryDataLinesTest;
					lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXMemoryDataLinesTest"); 						
					pUFXMemoryDataLinesTest = TESTDATALINES(lpfnGetProcessID);
					stMemStatus stMemStatusobj;
					stMemStatusobj = pUFXMemoryDataLinesTest();
					if(stMemStatusobj.dwStatus == UFX_DRAM_DATA_LINE_VER_PASS)
						printf("Testing UFXMemoryDataLinesTest API, Data Line test passed \n\n");
					else
						printf("Testing UFXMemoryDataLinesTest API, Data Line test failed \n\n");			
				}
				break;

			case 5:
				{
					TESTADDLINES pUFXMemoryAddressLinesTest;
					lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXMemoryAddressLinesTest"); 						
					pUFXMemoryAddressLinesTest = TESTDATALINES(lpfnGetProcessID);
					stMemStatusobj = pUFXMemoryAddressLinesTest();
					if(stMemStatusobj.dwStatus == UFX_DRAM_ADD_LINE_VER_PASS)
						printf("Testing UFXMemoryAddressLinesTest API, Address Line test passed \n\n");
					else
						printf("Testing UFXMemoryAddressLinesTest API, Address Line test failed \n\n");
				}
				break;
			
			case 6:
				{
					VERIFYFULLDRAM pUFXVerifyFullDRAM;
					lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXVerifyFullDRAM"); 						
					pUFXVerifyFullDRAM = VERIFYFULLDRAM(lpfnGetProcessID);

					printf("\nTesting full DRAM it will take several hours to finish... \n");
					stMemStatusobj = pUFXVerifyFullDRAM();
					if(stMemStatusobj.dwStatus == UFX_DRAM_VER_OVERALL_PASS)
					{
						printf("FULL DRAM test successfully completed \n");
					}
					else
						printf("FULL DRAM test failed \n");
				}
				break;

			case 7:
				{
					GETODTFREQ pUFXGetODTnFrequency;
					lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXGetODTnFrequency"); 						
					pUFXGetODTnFrequency = GETODTFREQ(lpfnGetProcessID);
					stMemoryParams stMemoryParamsObj;
					iResult = pUFXGetODTnFrequency(&stMemoryParamsObj);
					if(UFX_LIB_SUCCESS == iResult)	
						printf("Testing UFXGetODTnFrequency API, Retruned values:\nstMemoryParamsObj.ODT = %d\nstMemoryParamsObj.Freq = %d \n\n", stMemoryParamsObj.dwODTValue, stMemoryParamsObj.dwFrequency );
					else if(UFX_LIB_FAILURE == iResult)
						printf("UFX_LIB_FAILURE \n");
					else if(UFX_INVALID_ARG == iResult)
						printf("UFX_INVALID_ARG\n");
				}
			break;
			
			case 8:
				{
					SETODTFREQ pUFXSetODTnFrequency;
					lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXSetODTnFrequency"); 						
					pUFXSetODTnFrequency = SETODTFREQ(lpfnGetProcessID);
					stMemoryParams stMemoryParamsObj = {0};
					printf("Enter Frequency:  \n\t 1 -> 300MHz, \n\t 2 -> 400MHz, \n\t 3 -> 533MHz, \n\t 4 -> 667MHz \n");
					scanf("%d", &stMemoryParamsObj.dwFrequency);

					printf("Enter ODT Val:  \n\t 1 -> 75 Ohm, \n\t 2 -> 150 Ohm, \n\t 3 -> ODT Disabled \n");
					scanf("%d", &stMemoryParamsObj.dwODTValue);

					iResult = pUFXSetODTnFrequency(stMemoryParamsObj);
					if(UFX_LIB_SUCCESS == iResult)	
						printf(" Successfully set frequency and ODT Value \n\n");
					else if(UFX_FRM_WRK_INIT_FAIL == iResult)
						printf("UFX_FRM_WRK_INIT_FAIL \n");
					else if(UFX_HDWRE_INIT_FAIL == iResult)
						printf("UFX_HDWRE_INIT_FAIL\n");
					else if(UFX_DEV_COM_FAIL == iResult)
						printf("UFX_FRM_WRK_INIT_FAIL \n");
					else if(UFX_INVALID_ARG == iResult)
						printf("UFX_INVALID_ARG\n");
				}
			break;

			case 9:
			default:
				goto END;	
				break;
		}
	}while(1);

END:
	iResult = pUFXLibDeInit();
	
	/* Release the DLL */ 
 	FreeLibrary(hGetProcIDDLL);
		
	return;
}

typedef UFXLIB_RESULT (__cdecl *DRAWRECT)(UFXLIB_RECT_COLOR, stRect_t );
typedef UFXLIB_RESULT (__cdecl *PREFEREDRESOLUTION)(stResolution_t *);;
typedef UFXLIB_RESULT (__cdecl *DRAWPIC)(UCHAR *ucPath);
typedef UFXLIB_RESULT (__cdecl *GEINIT)(stResolution_t);
void Imaging()
{	
	int iOper = 0;
	stRect_t Rect;
	UFXLIB_RECT_COLOR eColor =  UFX_RED;
	stResolution_t stResolution = {0};
	stResolution_t stPrefereReso = {0};

	FARPROC lpfnGetProcessID;
	
	
	/* get handle to dll */ 
	HINSTANCE hGetProcIDDLL = NULL;
	hGetProcIDDLL = LoadLibrary(L"ufxutil.dll"); 
	if(NULL == hGetProcIDDLL)
	{
		printf("Failed to load ufxutil.dll library \n");
		return;
	}

	HMODULE hndl = NULL; 
	hndl = GetModuleHandle(L"ufxutil.dll");

	LIBINIT pUFXLibInit;
	LIBDEINIT pUFXLibDeInit;

	/* get pointer to the function in the dll*/ 
	lpfnGetProcessID = GetProcAddress(hndl,"UFXLibInit"); 
	pUFXLibInit = LIBINIT(lpfnGetProcessID);
	
	lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXLibDeInit"); 						
	pUFXLibDeInit = LIBDEINIT(lpfnGetProcessID);

	if(pUFXLibDeInit == NULL || pUFXLibInit == NULL)
	{
		printf("Failed to get address of function pUFXLibInit or pUFXLibDeInit \n");
		return;
	}

	/* Init Environment */
	printf("----------------Intializing library and Hornet Hardware----------------");
	int iResult = pUFXLibInit();
	if(UFX_LIB_MORE_THAN_ONE_DEV == iResult)
	{
		printf("UFX_LIB_MORE_THAN_ONE_DEV \n");
		return;
	}
	else if(UFX_FRM_WRK_INIT_FAIL == iResult)
	{
		printf("UFX_FRM_WRK_INIT_FAIL\n");
		return;
	}
	else if(UFX_HDWRE_INIT_FAIL == iResult)
	{
		printf("UFX_HDWRE_INIT_FAIL \n");
		return;
	}
	printf("--------------Library and Hornet Hardware init success------------ \n\n");

	GEINIT pUFXInitGE ;
	lpfnGetProcessID = GetProcAddress(hndl,"UFXInitGE"); 
	pUFXInitGE = GEINIT(lpfnGetProcessID);

	PREFEREDRESOLUTION pUFXPreferedResolution ;
	lpfnGetProcessID = GetProcAddress(hndl,"UFXPreferedResolution"); 
	pUFXPreferedResolution = PREFEREDRESOLUTION(lpfnGetProcessID);
	if(pUFXPreferedResolution == NULL || pUFXInitGE == NULL)
	{
		printf("Failed to get address of function UFXPreferedResolution or UFXInitGE \n");
		return;
	}

	iResult = pUFXPreferedResolution(&stPrefereReso);
	if( UFX_LIB_SUCCESS != iResult)
	{
		printf("Failed to get prefered resolution \n");
	}
	else
	{
		printf("Preferred Resolution & Refresh Rate of Monitor: \nHorizontal Pixel = %d \nVertical Pixels = %d \nRefreshRate  = %d\n\n", stPrefereReso.dwWidth , stPrefereReso.dwHeight ,stPrefereReso.dwFreuency);
/*
		iResult = pUFXInitGE(stPrefereReso);
		if(UFX_LIB_SUCCESS != iResult)
			printf("Failed to initialize GE on prefered resolution Err = %d\n",iResult);
*/
	}

	do{
		int iOper = 0;
		printf("-------- Image Test Menu -----------\n");
		printf("\t1. Draw rectangle operation\n");
		printf("\t2. Image draw operation \n");
		printf("\t3. Exit \n");
		printf("------------------------------------ \n");
		printf("Option:: ");
		scanf("%d",&iOper);
		if(iOper == 0)
			break;

		switch(iOper)
		{
			case 1:
				{
					DRAWRECT pUFXDrawRect;
					pUFXDrawRect = NULL;
					
					printf("----------------Select Monitor Resolution ----------------\n");
					printf("1. Draw rectangle at prefered resolution \n");
					printf("2. Draw rectangle at 640x480 @ 60\n");
					printf("3. Draw rectangle at 800x600 @ 60\n");
					printf("4. Draw rectangle at 1280x1024 @ 60\n");
					printf("5. Draw rectangle at 1440x900 @ 60\n");
					printf("6. Draw rectangle at 1920x1080 @ 60\n");
					printf("-------------------------------------------------------------\n");
					iOper = 0;
					printf("Option:: ");
					scanf("%d",&iOper);
					switch(iOper)
					{
						case 1:
							{
								iResult = pUFXPreferedResolution(&stPrefereReso);
								if( UFX_LIB_SUCCESS != iResult)
								{
									printf("Failed to get prefered resolution \n\n");
									continue;
								}
								printf("\nPreferred Resolution & Refresh Rate of Monitor: \nHorizontal Pixel = %d \nVertical Pixels = %d \nRefreshRate  = %d\n", stPrefereReso.dwWidth , stPrefereReso.dwHeight ,stPrefereReso.dwFreuency);

								printf("----------------Initializing GE to %dx%d @ %d----------------\n",stPrefereReso.dwWidth,stPrefereReso.dwHeight,stPrefereReso.dwFreuency );
								iResult = pUFXInitGE(stPrefereReso);
								if(UFX_LIB_SUCCESS != iResult)
									printf("----------------Failed to initialize GE to %dx%d @ %d----------------\n\n",stPrefereReso.dwWidth,stPrefereReso.dwHeight,stPrefereReso.dwFreuency );									
								else
									printf("----------------GE initialization to %dx%d @ %d success----------------\n\n",stPrefereReso.dwWidth,stPrefereReso.dwHeight,stPrefereReso.dwFreuency );
							}
							break;
						case 2:
							{
								iResult = pUFXPreferedResolution(&stPrefereReso);
								if( UFX_LIB_SUCCESS != iResult)
								{
									printf("Failed to get prefered resolution \n\n");
									continue;
								}
								else
								{
									if(stPrefereReso.dwHeight < 480 || stPrefereReso.dwWidth < 640 )
									{
										printf("Monitor donot support 640x480 @ 60  \n\n");
										continue;
									}
									
									stPrefereReso.dwHeight = 480;
									stPrefereReso.dwWidth = 640;
									stPrefereReso.dwFreuency = 60;
								
									printf("----------------Initializing GE to 640x480 @ 60 ----------------\n");
									iResult = pUFXInitGE(stPrefereReso);
									if(UFX_LIB_SUCCESS != iResult)
										printf("----------------Failed to initialize GE on resolution 640x480 @ 60, Err = %d----------------\n",iResult);
									else
										printf("----------------GE initialization to 640x480 @ 60 success ----------------\n\n");
								}
							}
							break;
						case 3:
							{
								iResult = pUFXPreferedResolution(&stPrefereReso);
								if( UFX_LIB_SUCCESS != iResult)
								{
									printf("Failed to get prefered resolution \n\n");
									continue;
								}
								else
								{
									if(stPrefereReso.dwHeight < 600 || stPrefereReso.dwWidth < 800 )
									{
										printf("Monitor donot support 800x600 @ 60 \n\n");
										continue;
									}
									stPrefereReso.dwHeight = 600;
									stPrefereReso.dwWidth = 800;
									stPrefereReso.dwFreuency = 60;
								
									printf("----------------Initializing GE to 800x600 @ 60 ---------------- \n");
									iResult = pUFXInitGE(stPrefereReso);
									if(UFX_LIB_SUCCESS != iResult)
										printf("----------------Failed to initialize GE on resolution 800x600 @ 60, Err = %d----------------\n",iResult);
									else
										printf("----------------GE initialization to 800x600 @ 60 success ----------------\n\n");
								}
							}
							break;
						case 4:
							{
								iResult = pUFXPreferedResolution(&stPrefereReso);
								if( UFX_LIB_SUCCESS != iResult)
								{
									printf("Failed to get prefered resolution \n");
									continue;
								}
								else
								{
									if(stPrefereReso.dwHeight < 1024 || stPrefereReso.dwWidth < 1280 )
									{
										printf("Monitor donot support 1280x1024 @ 60 \n\n");
										continue;
									}

									stPrefereReso.dwHeight = 1024;
									stPrefereReso.dwWidth = 1280;
									stPrefereReso.dwFreuency = 60;

									printf("----------------Initializing GE to 1280x1024 @ 60 ----------------\n");
									iResult = pUFXInitGE(stPrefereReso);
									if(UFX_LIB_SUCCESS != iResult)
										printf("----------------Failed to initialize GE on resolution 1280x1024 @ 60, Err = %d----------------\n",iResult);
									else
										printf("----------------GE initialization to 1280x1024 @ 60 success ----------------\n\n");
								}
							}
							break;
						case 5:
							{
								iResult = pUFXPreferedResolution(&stPrefereReso);
								if( UFX_LIB_SUCCESS != iResult)
								{
									printf("Failed to get prefered resolution \n");
									continue;
								}
								else
								{
									if(stPrefereReso.dwHeight < 900 || stPrefereReso.dwWidth < 1440 )
									{
										printf("Monitor donot support 1440x900 @ 60 \n\n");
										continue;
									}

									stPrefereReso.dwHeight = 900;
									stPrefereReso.dwWidth = 1440;
									stPrefereReso.dwFreuency = 60;
									printf("----------------Initializing GE to 1440x900 @ 60 ----------------\n");
									iResult = pUFXInitGE(stPrefereReso);
									if(UFX_LIB_SUCCESS != iResult)
										printf("----------------Failed to initialize GE on resolution 1440x900 @ 60, Err = %d----------------\n",iResult);
									else
										printf("----------------GE initialization to 1440x900 @ 60 success ----------------\n\n");
								}
							}
							break;
						case 6:
							{
								iResult = pUFXPreferedResolution(&stPrefereReso);
								if( UFX_LIB_SUCCESS != iResult)
								{
									printf("Failed to get prefered resolution \n\n");
									continue;
								}
								else
								{
									if(stPrefereReso.dwHeight < 1080 || stPrefereReso.dwWidth < 1920 )
									{
										printf("Monitor donot support 1920x1080 @ 60 \n\n");
										continue;
									}

									stPrefereReso.dwHeight = 1080;
									stPrefereReso.dwWidth = 1920;
									stPrefereReso.dwFreuency = 60;
									printf("----------------Initializing GE to 1920x1080 @ 60 ----------------\n");
									iResult = pUFXInitGE(stPrefereReso);
									if(UFX_LIB_SUCCESS != iResult)
										printf("----------------Failed to initialize GE on resolution 1920x1080 @ 60, Err = %d----------------\n",iResult);
									else
										printf("----------------GE initialization to 1920x1080 @ 60 success ----------------\n\n");
								}
							}
							break;
						default:
							continue;
							break;
					}


#if 0					
					PREFEREDRESOLUTION pUFXPreferedResolution ;
					lpfnGetProcessID = GetProcAddress(hndl,"UFXPreferedResolution"); 
					pUFXPreferedResolution = PREFEREDRESOLUTION(lpfnGetProcessID);
	
					iResult = pUFXPreferedResolution(&stResolution);
					printf(" Preferred Resolution & Timing of Monitor: \n\t Horizontal Pixel = %d \n\t Vertical Pixels = %d \n\t RefreshRate  = %d \n\n", stResolution.dwWidth , stResolution.dwHeight ,stResolution.dwFreuency);
#endif				
					
					lpfnGetProcessID = GetProcAddress(hndl,"UFXDrawRect"); 
					pUFXDrawRect = DRAWRECT(lpfnGetProcessID);

					if(pUFXDrawRect != NULL )
					{
						Rect.iHeight = stResolution.dwHeight - 100;
						Rect.iWidth = stResolution.dwWidth - 100;
						Rect.iXpos = Rect.iYpos = 0;
					
						int color;
						printf(" Enter Rectangle Color: \t\n 0 -> Red \t\n 1 -> Green \t\n 2 -> Blue \t\n 3 -> White \t\n 4 -> Black\n");
						scanf("%d", &color);
						eColor = (UFXLIB_RECT_COLOR)color;
						int dwPos;
						dwPos = 0;
						printf(" \nEnter starting co-ordinates for Rectangle:");
						printf(" \nX - Pos : ");
						scanf("%d", &dwPos);
						Rect.iXpos = dwPos;
						printf(" \nY - Pos : ");
						scanf("%d",&dwPos);
						Rect.iYpos = dwPos;

						printf(" \nEnter Width, 'X-Pos + Width' should not greater than %d \n",stPrefereReso.dwWidth);
						printf(" Width : ");
						scanf("%d", &dwPos);
						Rect.iWidth = dwPos;
						printf(" \nEnter Height, 'Y-Pos + Hight' should not greater than %d \n",stPrefereReso.dwHeight);
						printf(" Height : ");
						scanf("%d", &dwPos);
						Rect.iHeight = dwPos;

						iResult = pUFXDrawRect(eColor,Rect);
						if(UFX_LIB_SUCCESS == iResult)
							printf("Rectangle drawn successfully. \n\n");
						else if(UFX_DEV_COM_FAIL == iResult)
							printf("UFX_DEV_COM_FAIL \n");
						else if(UFX_LIB_FAILURE == iResult)
							printf("UFX_LIB_FAILURE\n");
						else if(UFX_INVALID_ARG == iResult)
							printf("UFX_INVALID_ARG \n");
						else if(UFX_INVALID_DRAW_REQ == iResult)
							printf("UFX_INVALID_DRAW_REQ \n");
						else if(UFX_GE_INIT_FAILED == iResult)
							printf("UFX_GE_INIT_FAILED \n");
						else if(UFX_LIB_FAILURE == iResult)
						{
							printf("\n GE not initialized properly, Use case 1 to initialize GE. \n");
						}
						else if(UFX_HDWRE_INIT_FAIL == iResult)
							printf("UFX_HDWRE_INIT_FAIL \n");
					}
					else
					{
						printf(" \n Failed get address UFXInitGE or UFXDrawRectfrom library. Library not loaded correctly \n");
					}
				}
				break;

			case 2:
				{
					CHAR *ucPath = "C:\\SMSCWorkPlace\\Project\\Hornet\\Hornet_Project\\Code\\Images\\1280x1024_7.bmp";
					CHAR ucPath1[512] = {0}; 
					DRAWPIC pUFXDrawPic;
					pUFXDrawPic = NULL;
#if 0					
					PREFEREDRESOLUTION pUFXPreferedResolution ;
					lpfnGetProcessID = GetProcAddress(hndl,"UFXPreferedResolution"); 
					pUFXPreferedResolution = PREFEREDRESOLUTION(lpfnGetProcessID);
	
					iResult = pUFXPreferedResolution(&stResolution);
					printf(" Preferred Resoultion & Timing of Monitor: \n\t Horizontal Pixel = %d \n\t Vertical Pixels = %d \n\t RefreshRate  = %d \n\n", stResolution.dwWidth , stResolution.dwHeight ,stResolution.dwFreuency);
#endif					
					
					printf("----------------Select Monitor Resolution ----------------\n");
					printf("1. Draw image at prefered resolution \n");
					printf("2. Draw image at 640x480 @ 60\n");
					printf("3. Draw image at 800x600 @ 60\n");
					printf("4. Draw image at 1280x1024 @ 60\n");
					printf("5. Draw image at 1440x900 @ 60\n");
					printf("6. Draw image at 1920x1080 @ 60\n");
					printf("-------------------------------------------------------------\n");
					iOper = 0;
					printf("Option:: ");
					scanf("%d",&iOper);
					

					switch(iOper)
					{
						case 1:
							{
								iResult = pUFXPreferedResolution(&stPrefereReso);
								if( UFX_LIB_SUCCESS != iResult)
								{
									printf("Failed to get prefered resolution \n\n");
									continue;
								}
								printf("\nPreferred Resolution & Refresh Rate of Monitor: \nHorizontal Pixel = %d \nVertical Pixels = %d \nRefreshRate  = %d\n", stPrefereReso.dwWidth , stPrefereReso.dwHeight ,stPrefereReso.dwFreuency);

								printf("----------------Initializing GE to %dx%d @ %d----------------\n",stPrefereReso.dwWidth,stPrefereReso.dwHeight,stPrefereReso.dwFreuency );
								iResult = pUFXInitGE(stPrefereReso);
								if(UFX_LIB_SUCCESS != iResult)
									printf("----------------Failed to initialize GE to %dx%d @ %d----------------\n\n",stPrefereReso.dwWidth,stPrefereReso.dwHeight,stPrefereReso.dwFreuency );									
								else
									printf("----------------GE initialization to %dx%d @ %d success----------------\n\n",stPrefereReso.dwWidth,stPrefereReso.dwHeight,stPrefereReso.dwFreuency );
							}
							break;
						case 2:
							{
								iResult = pUFXPreferedResolution(&stPrefereReso);
								if( UFX_LIB_SUCCESS != iResult)
								{
									printf("Failed to get prefered resolution \n\n");
									continue;
								}
								else
								{
									if(stPrefereReso.dwHeight < 480 || stPrefereReso.dwWidth < 640 )
									{
										printf("Monitor donot support 640x480 @ 60  \n\n");
										continue;
									}
									
									stPrefereReso.dwHeight = 480;
									stPrefereReso.dwWidth = 640;
									stPrefereReso.dwFreuency = 60;
								
									printf("----------------Initializing GE to 640x480 @ 60 ----------------\n");
									iResult = pUFXInitGE(stPrefereReso);
									if(UFX_LIB_SUCCESS != iResult)
										printf("----------------Failed to initialize GE on resolution 640x480 @ 60, Err = %d----------------\n",iResult);
									else
										printf("----------------GE initialization to 640x480 @ 60 success ----------------\n\n");
								}
							}
							break;
						case 3:
							{
								iResult = pUFXPreferedResolution(&stPrefereReso);
								if( UFX_LIB_SUCCESS != iResult)
								{
									printf("Failed to get prefered resolution \n\n");
									continue;
								}
								else
								{
									if(stPrefereReso.dwHeight < 600 || stPrefereReso.dwWidth < 800 )
									{
										printf("Monitor donot support 800x600 @ 60 \n\n");
										continue;
									}
									stPrefereReso.dwHeight = 600;
									stPrefereReso.dwWidth = 800;
									stPrefereReso.dwFreuency = 60;
								
									printf("----------------Initializing GE to 800x600 @ 60 ---------------- \n");
									iResult = pUFXInitGE(stPrefereReso);
									if(UFX_LIB_SUCCESS != iResult)
										printf("----------------Failed to initialize GE on resolution 800x600 @ 60, Err = %d----------------\n",iResult);
									else
										printf("----------------GE initialization to 800x600 @ 60 success ----------------\n\n");
								}
							}
							break;
						case 4:
							{
								iResult = pUFXPreferedResolution(&stPrefereReso);
								if( UFX_LIB_SUCCESS != iResult)
								{
									printf("Failed to get prefered resolution \n");
									continue;
								}
								else
								{
									if(stPrefereReso.dwHeight < 1024 || stPrefereReso.dwWidth < 1280 )
									{
										printf("Monitor donot support 1280x1024 @ 60 \n\n");
										continue;
									}

									stPrefereReso.dwHeight = 1024;
									stPrefereReso.dwWidth = 1280;
									stPrefereReso.dwFreuency = 60;

									printf("----------------Initializing GE to 1280x1024 @ 60 ----------------\n");
									iResult = pUFXInitGE(stPrefereReso);
									if(UFX_LIB_SUCCESS != iResult)
										printf("----------------Failed to initialize GE on resolution 1280x1024 @ 60, Err = %d----------------\n",iResult);
									else
										printf("----------------GE initialization to 1280x1024 @ 60 success ----------------\n\n");
								}
							}
							break;
						case 5:
							{
								iResult = pUFXPreferedResolution(&stPrefereReso);
								if( UFX_LIB_SUCCESS != iResult)
								{
									printf("Failed to get prefered resolution \n");
									continue;
								}
								else
								{
									if(stPrefereReso.dwHeight < 900 || stPrefereReso.dwWidth < 1440 )
									{
										printf("Monitor donot support 1440x900 @ 60 \n\n");
										continue;
									}

									stPrefereReso.dwHeight = 900;
									stPrefereReso.dwWidth = 1440;
									stPrefereReso.dwFreuency = 60;
									printf("----------------Initializing GE to 1440x900 @ 60 ----------------\n");
									iResult = pUFXInitGE(stPrefereReso);
									if(UFX_LIB_SUCCESS != iResult)
										printf("----------------Failed to initialize GE on resolution 1440x900 @ 60, Err = %d----------------\n",iResult);
									else
										printf("----------------GE initialization to 1440x900 @ 60 success ----------------\n\n");
								}
							}
							break;
						case 6:
							{
								iResult = pUFXPreferedResolution(&stPrefereReso);
								if( UFX_LIB_SUCCESS != iResult)
								{
									printf("Failed to get prefered resolution \n\n");
									continue;
								}
								else
								{
									if(stPrefereReso.dwHeight < 1080 || stPrefereReso.dwWidth < 1920 )
									{
										printf("Monitor donot support 1920x1080 @ 60 \n\n");
										continue;
									}

									stPrefereReso.dwHeight = 1080;
									stPrefereReso.dwWidth = 1920;
									stPrefereReso.dwFreuency = 60;
									printf("----------------Initializing GE to 1920x1080 @ 60 ----------------\n");
									iResult = pUFXInitGE(stPrefereReso);
									if(UFX_LIB_SUCCESS != iResult)
										printf("----------------Failed to initialize GE on resolution 1920x1080 @ 60, Err = %d----------------\n",iResult);
									else
										printf("----------------GE initialization to 1920x1080 @ 60 success ----------------\n\n");
								}
							}
							break;
						default:
							continue;
							break;
					}

					printf("\nPlease enter Image path(Image Resolution should match selected Monitor Resolution):  ");
					scanf("%s", ucPath1);

					lpfnGetProcessID = GetProcAddress(hndl,"UFXDrawPic"); 
					pUFXDrawPic = DRAWPIC(lpfnGetProcessID);
	
					iResult = pUFXDrawPic((UCHAR*) ucPath1);
					if(UFX_LIB_SUCCESS != iResult)
					{						
						pUFXLibDeInit();
						pUFXLibInit();
					}
					printf("\n");

					if(UFX_LIB_SUCCESS == iResult)
						printf("Image %s drawn successfully. \n\n",ucPath1);
					else if(UFX_LIB_FAILURE == iResult)
					{
						printf("\n GE not initialized properly, Use case 1 to initialize GE. \n");
					}
					else if(UFX_HDWRE_INIT_FAIL == iResult)
						printf("UFX_HDWRE_INIT_FAIL \n");
					else if(UFX_DEV_COM_FAIL == iResult)
						printf("UFX_DEV_COM_FAIL \n");
					else if(UFX_INVALID_ARG == iResult)
						printf("UFX_INVALID_ARG \n");
					else if(UFX_EDID_PARSE_ERR == iResult)
						printf("UFX_EDID_PARSE_ERR \n");
					else if(UFX_GE_INIT_FAILED == iResult)
					{
						printf("\n GE not initialized properly. \n");
					}
					else if(UFX_RESOLUTION_MISMATCH == iResult)
						printf("UFX_RESOLUTION_MISMATCH \n");
					else if(UFX_FILE_NOT_FOUND == iResult)
						printf("UFX_FILE_NOT_FOUND \n");
					else if(UFX_WRONG_FILE_FORMAT == iResult)
						printf("UFX_WRONG_FILE_FORMAT \n");						
				}
				break;
			case 3:
			default:
				goto END;
				break;
		}
	}while(1);

END:
	iResult = pUFXLibDeInit();
	
	/* Release the DLL */ 
 	FreeLibrary(hGetProcIDDLL);
		
	return;
}

typedef UFXLIB_RESULT (__cdecl *LIBINIT)();
typedef UFXLIB_RESULT (__cdecl *LIBDEINIT)();

typedef UFXLIB_RESULT (__cdecl *HDMIPHYENABLE)(HDMI_SW_CFG_DATA *pHDMI_SW_CFG_DATAObj);
typedef UFXLIB_RESULT (__cdecl *HDMIPHYDISABLE)();
typedef UFXLIB_RESULT (__cdecl *HDMIMUTE)();
typedef UFXLIB_RESULT (__cdecl *HDMIUNMUTE)();

void HdmiSound()
{	
	int iResult ;
	FARPROC lpfnGetProcessID;
	/* get handle to dll */ 
	HINSTANCE hGetProcIDDLL = NULL;
	hGetProcIDDLL = LoadLibrary(L"ufxutil.dll"); 
	if(NULL == hGetProcIDDLL)
	{
		printf("Failed to load ufxutil.dll library \n");
		return;
	}

	HMODULE hndl = NULL; 
	hndl = GetModuleHandle(L"ufxutil.dll");

	LIBINIT pUFXLibInit;
	LIBDEINIT pUFXLibDeInit;

	/* get pointer to the function in the dll*/ 
	lpfnGetProcessID = GetProcAddress(hndl,"UFXLibInit"); 
	pUFXLibInit = LIBINIT(lpfnGetProcessID);
	
	lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXLibDeInit"); 						
	pUFXLibDeInit = LIBDEINIT(lpfnGetProcessID);

	/* Init Environment */
	iResult = pUFXLibInit();
	if(UFX_LIB_MORE_THAN_ONE_DEV == iResult)
	{
		printf("UFX_LIB_MORE_THAN_ONE_DEV \n");
		return;
	}
	else if(UFX_FRM_WRK_INIT_FAIL == iResult)
		printf("UFX_FRM_WRK_INIT_FAIL\n");
	else if(UFX_HDWRE_INIT_FAIL == iResult)
		printf("UFX_HDWRE_INIT_FAIL \n");
	
	printf("********* HDMI Sound Function testing ******** \n\n");

	int iOper = 0;

	/**************************** Reading/Writing to/From EEPROM: Ends **************/ 
	do{
		
		printf("--------------------- HDMI Sound Test Menu ---------------------\n");
		printf("\t1. Enable HDMI Sound and Video \n");
		printf("\t2. Disable HDMI Sound and Video \n");
		printf("\t3. Exit \n");
		printf("---------------------------------------------------------------\n");
		//printf("\t4. Mute HDMI Sound \n");
		//printf("\t5. Unmute HDMI Sound  \n");
		printf("Options:");
		scanf("%d",&iOper);
		printf("\n");
		switch(iOper)
		{
			case 1:
				{
					HDMI_SW_CFG_DATA HDMI_SW_CFG_DATAObj;
					HDMIPHYENABLE pUFXHdmiPhyEnable;
#if 0					
					printf("HDMI initialize require following parameters from user \n\n");
					printf("1. Display controller Port\n2. HDMI Audio info\n3. Sampling Frequency\n4. MCLK Input Mode \n5. Infoframe Suppression \n6. SVAL \n\n");
					printf("\n\n********** Select the option **************\n");
					printf("\t1. Use default values \n");
					printf("\t2. User specified\n");
					printf("Option:");
					scanf("%d",&iOper);
#endif
					printf("\n");

					iOper = 1;

					switch(iOper)
					{
						case 2:
							printf("------>  Not implemented, Initializing with default values \n");
						
						case 1:
						default:
							printf("Initializing HDMI with default values \n");
							
							printf("Here are default values: \n");
							HDMI_SW_CFG_DATAObj.dwPortType = DC_PORT_HDMI_AV;
							printf("Display Controller Port set to 'DC_PORT_HDMI_AV'\n");
							
							HDMI_SW_CFG_DATAObj.bAudio = AUDIO_I2S0;
							printf("Audio set to 'AUDIO_I2S0'\n");

							HDMI_SW_CFG_DATAObj.bAudioFS = I2S_FS_AUTO;
							printf("Audio Sampling Frequency set to 'I2S_FS_AUTO'\n");

							HDMI_SW_CFG_DATAObj.bMCLK = 2;
							printf("MCLK mode set to '256 * Fs'\n");

							HDMI_SW_CFG_DATAObj.bN_SVAL = 0;
							printf("N_SVAL set to 'Default'\n");

							HDMI_SW_CFG_DATAObj.bSuppressAVI = 0;
							printf("Infro frame suppression set to Disable\n");
							printf("-----------------------------------------------------------\n\n");
							break;
					}

					pUFXHdmiPhyEnable = (HDMIPHYENABLE)GetProcAddress(HMODULE (hGetProcIDDLL),"UFXHdmiPhyEnable"); 						
					printf("\n\n-----------------------Init HDMI------------------------------------\n");
					iResult = pUFXHdmiPhyEnable(&HDMI_SW_CFG_DATAObj);		
					printf("\n-----------------------Init HDMI done------------------------------------\n\n");
					if(UFX_LIB_SUCCESS == iResult)	
						printf("HDMI Sound enabled successfully \n");
					else if(UFX_LIB_MORE_THAN_ONE_DEV == iResult)
						printf("UFX_LIB_MORE_THAN_ONE_DEV \n");
					else if(UFX_FRM_WRK_INIT_FAIL == iResult)
						printf("UFX_FRM_WRK_INIT_FAIL\n");
					else if(UFX_HDWRE_INIT_FAIL == iResult)
						printf("UFX_HDWRE_INIT_FAIL \n");				
				}
				break;

			case 2:
				{	
					HDMIPHYDISABLE pUFXHdmiPhyDisable;
					pUFXHdmiPhyDisable = (HDMIPHYDISABLE)GetProcAddress(HMODULE (hGetProcIDDLL),"UFXHdmiPhyDisable"); 						
					iResult = pUFXHdmiPhyDisable();		
					if(UFX_LIB_SUCCESS == iResult)	
						printf("HDMI Sound enabled successfully \n");
					else if(UFX_LIB_MORE_THAN_ONE_DEV == iResult)
						printf("UFX_LIB_MORE_THAN_ONE_DEV \n");
					else if(UFX_FRM_WRK_INIT_FAIL == iResult)
						printf("UFX_FRM_WRK_INIT_FAIL\n");
					else if(UFX_HDWRE_INIT_FAIL == iResult)
						printf("UFX_HDWRE_INIT_FAIL \n");				
					
				}
				break;

			case 4:
				{
					HDMIMUTE pUFXHdmiMute;
					pUFXHdmiMute = (HDMIMUTE)GetProcAddress(HMODULE (hGetProcIDDLL),"UFXHdmiMute"); 						
					iResult = pUFXHdmiMute();		
					if(UFX_LIB_SUCCESS == iResult)	
						printf("HDMI Sound enabled successfully \n");
					else if(UFX_LIB_MORE_THAN_ONE_DEV == iResult)
						printf("UFX_LIB_MORE_THAN_ONE_DEV \n");
					else if(UFX_FRM_WRK_INIT_FAIL == iResult)
						printf("UFX_FRM_WRK_INIT_FAIL\n");
					else if(UFX_HDWRE_INIT_FAIL == iResult)
						printf("UFX_HDWRE_INIT_FAIL \n");			
				}
				break;

			case 5:
				{
					HDMIUNMUTE pUFXHdmiUnMute;
					pUFXHdmiUnMute = (HDMIUNMUTE)GetProcAddress(HMODULE (hGetProcIDDLL),"UFXHdmiUnMute"); 						
					iResult = pUFXHdmiUnMute();		
					if(UFX_LIB_SUCCESS == iResult)	
						printf("HDMI Sound enabled successfully \n");
					else if(UFX_LIB_MORE_THAN_ONE_DEV == iResult)
						printf("UFX_LIB_MORE_THAN_ONE_DEV \n");
					else if(UFX_FRM_WRK_INIT_FAIL == iResult)
						printf("UFX_FRM_WRK_INIT_FAIL\n");
					else if(UFX_HDWRE_INIT_FAIL == iResult)
						printf("UFX_HDWRE_INIT_FAIL \n");			
				}
				break;
			case 3:
			default:
				goto END;	
				break;
		}
	}while( iOper != 3 );

END:
	iResult = pUFXLibDeInit();
	
	/* Release the DLL */
 	FreeLibrary(hGetProcIDDLL);
	
	return;
}

typedef UFXLIB_RESULT (__cdecl *RESETDEVICE)();

int ResetDevice()
{
	int iResult ;
	FARPROC lpfnGetProcessID;
	/* get handle to dll */ 
	HINSTANCE hGetProcIDDLL = NULL;
	hGetProcIDDLL = LoadLibrary(L"ufxutil.dll"); 
	if(NULL == hGetProcIDDLL)
	{
		printf("Failed to load ufxutil.dll library \n");
		return 0;
	}

	HMODULE hndl = NULL; 
	hndl = GetModuleHandle(L"ufxutil.dll");

	LIBINIT pUFXLibInit;
	LIBDEINIT pUFXLibDeInit;
	RESETDEVICE pUFXResetDevice;

	/* get pointer to the function in the dll*/ 
	lpfnGetProcessID = GetProcAddress(hndl,"UFXLibInit"); 
	pUFXLibInit = LIBINIT(lpfnGetProcessID);
	
	lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXLibDeInit"); 						
	pUFXLibDeInit = LIBDEINIT(lpfnGetProcessID);

	/* Init Environment */
	iResult = pUFXLibInit();
	if(UFX_LIB_MORE_THAN_ONE_DEV == iResult)
	{
		printf("UFX_LIB_MORE_THAN_ONE_DEV \n");
		return 0;
	}
	else if(UFX_FRM_WRK_INIT_FAIL == iResult)
		printf("UFX_FRM_WRK_INIT_FAIL\n");
	else if(UFX_HDWRE_INIT_FAIL == iResult)
		printf("UFX_HDWRE_INIT_FAIL \n");

	lpfnGetProcessID = GetProcAddress(HMODULE (hGetProcIDDLL),"UFXResetDevice"); 						
	pUFXResetDevice = LIBDEINIT(lpfnGetProcessID);

	iResult = pUFXResetDevice();
	if(iResult == UFX_LIB_SUCCESS)
		printf("Successfully reset the device \n");
	else
		printf("Failed to reset device \n");
END:
	iResult = pUFXLibDeInit();
	
	/* Release the DLL */
 	FreeLibrary(hGetProcIDDLL);
	
	return 0;
}

int _tmain(int argc, _TCHAR* argv[])
{
	int iOper = 0;
	printf("------------------------------------------------------------------------------------\n");
	printf("Note: This application is a test application for ufxutil library. Its primary objective is to show usage of APIs \n\
provided in the library as support to calling application programmers. Messages and results may be ad hoc in some cases.\n");
	printf("------------------------------------------------------------------------------------\n\n");
	do{
		iOper = 0;
		printf("-------- UFX Library Test Menu -----------\n");
		printf("\t1. EEPROM Functions\n");
		printf("\t2. DRAM functions\n");
		printf("\t3. Image/Rectangle draw operations \n");
		printf("\t4. HDMI Sound functions\n");
		printf("\t5. Reset Device \n");
		printf("\t6. Exit \n");
		printf("------------------------------------------ \n");
		printf("Option:: ");
		scanf("%d",&iOper);
		printf("\n");
		if(iOper == 0)
		{
			printf("Invalid operation, Application exiting! \n Press enter....");		 
		}
		switch(iOper)
		{
			case 1:
				EepromTesting();
				break;
			case 2:
				DramTesting();
				break;
			case 3:
				Imaging();
				break;
			case 4:
				HdmiSound();
				break;
			case 5:
				printf("Reset device will reset the device and reload the EEPROM content, Device will temporarily disappear from\
 device manager and appear again with new EEPROM contents\nEnter 1 to continue");
				scanf("%d",&iOper);
				if(iOper == 1 )
					ResetDevice();
				break;
			case 6:				
			default:
				goto END;
				break;
		}
	}while(1);

END:
	getchar();

	return 0;
}
